"""

Test for suggestionlist objects

Meant to be subclassed

"""
from django.contrib.auth.models import User
from django.core import serializers
from django.test import TestCase

from installations import models as installations_models

from utils import tests as utils_tests
from utils.versioning.models import WorkVersion, Version

class SuggestionlistTest(TestCase):
    fixtures = ['dummyusers', 'installations']
##    sc_model = Scenario
##    pl_model = ProtectionLayer
##    me_model = Measure
##    ap_model = AttentionPoint
##    sl_model = Suggestionlist
##    namespace = "degradations"

    def setUp(self):
        self.setPermissions()
        defaultuser = User.objects.get(pk=1)

        self.suggestionlist = self.sl_model(name='SL')
        self.suggestionlist.save(defaultuser)
        self.slscenario = self.sc_model(name='sl scenario', suggestionlist=self.suggestionlist)
        self.slscenario.save(defaultuser)
        self.emptysuggestionlist = self.sl_model(name='empty SL')
        self.emptysuggestionlist.save(defaultuser)

        self.scope = installations_models.Scope.objects.get(pk=1)

    def test_model(self):
        # test scope and suggestionlist attributes
        self.assertEquals(self.slscenario.scope, None)
        self.assertEquals(self.slscenario.suggestionlist, self.suggestionlist)

    def test_model_delete(self):
        """
        test delete suggestionlist deletes all scenarios and subobjects
        """
        sl_c = self.sl_model.objects.count()
        sc_c = self.sc_model.objects.count()
        tree_c = self.tree_model.objects.count()
        ev_c = self.ev_model.objects.count()
        pl_c = self.pl_model.objects.count()
        me_c = self.me_model.objects.count()
        plme_c = self.plme_model.objects.count()
        ap_c = self.ap_model.objects.count()

        defaultuser = User.objects.get(pk=1)
        suggestionlist = self.sl_model(name='SL 2')
        suggestionlist.save(defaultuser)
        scenario = self.sc_model(name='scenario')
        scenario.suggestionlist=suggestionlist
        scenario.save(defaultuser)
        event = self.ev_model(name='C1')
        event.save(defaultuser, parent=None, treeelementkwargs={'scenario': scenario})
        protectionlayer = self.pl_model(name='PL')
        protectionlayer.save(defaultuser, parent=None, treeelementkwargs={'scenario': scenario})

        self.assertEquals(self.sl_model.objects.count(), sl_c+1)
        self.assertEquals(self.sc_model.objects.count(), sc_c+1)
        self.assertEquals(self.tree_model.objects.count(), tree_c+2)
        self.assertEquals(self.ev_model.objects.count(), ev_c+1)
        self.assertEquals(self.pl_model.objects.count(), pl_c+1)
        self.assertEquals(self.me_model.objects.count(), me_c)
        self.assertEquals(self.plme_model.objects.count(), plme_c)
        self.assertEquals(self.ap_model.objects.count(), ap_c)

        self.sl_model.objects.get(pk=suggestionlist.pk).delete(defaultuser)

        self.assertEquals(self.sl_model.objects.count(), sl_c)
        self.assertEquals(self.sc_model.objects.count(), sc_c)
        self.assertEquals(self.tree_model.objects.count(), tree_c)
        self.assertEquals(self.ev_model.objects.count(), ev_c)
        self.assertEquals(self.pl_model.objects.count(), pl_c)
        self.assertEquals(self.me_model.objects.count(), me_c)
        self.assertEquals(self.plme_model.objects.count(), plme_c)
        self.assertEquals(self.ap_model.objects.count(), ap_c)

    def test_model_dump_suggestionlist(self):
        dump = self.suggestionlist.dump()
        objects=serializers.deserialize('json', dump)
        self.assertEquals(len(list(objects)), 2)

    def test_publish(self):
        defaultuser = User.objects.get(pk=1)

        sl_c = self.sl_model.objects.count()
        sc_c = self.sc_model.objects.count()
        self.assertNotEquals(self.sl_model.objects.filter(published_dirty=True).count(), 0)
        self.assertNotEquals(self.sc_model.objects.filter(published_dirty=True).count(), 0)

        # publish
        v1 = Version(name='v1', published=True)
        v1.save(defaultuser)

        self.assertEquals(self.sl_model.objects.count(), 2 * sl_c)
        self.assertEquals(self.sc_model.objects.count(), 2 * sc_c)
        self.assertEquals(self.sl_model.objects.filter(published_dirty=True).count(), 0)
        self.assertEquals(self.sc_model.objects.filter(published_dirty=True).count(), 0)

        # change something
        self.slscenario = self.sc_model.objects.get_versionedobject(v_id=self.slscenario.v_id, version=WorkVersion)
        self.slscenario.name = u'other name'
        self.slscenario.save(defaultuser)

        self.assertEquals(self.sl_model.objects.filter(published_dirty=True).count(), 0)
        self.assertEquals(self.sc_model.objects.filter(published_dirty=True).count(), 1)

        v2 = Version(name='v2', published=True)
        v2.save(defaultuser, propagate=False)
        self.suggestionlist.publish(defaultuser, v2)

        self.assertEquals(self.sl_model.objects.count(), 2 * sl_c)
        self.assertEquals(self.sc_model.objects.count(), 2 * sc_c + 1)
        self.assertEquals(self.sl_model.objects.filter(published_dirty=True).count(), 0)
        self.assertEquals(self.sc_model.objects.filter(published_dirty=True).count(), 0)

        # let's check the scenario name in the published v2

        sc = self.sc_model.objects.get_versionedobject(v_id=self.slscenario.v_id, version=v2)
        self.assertEquals(sc.name, u'other name')

        # let's check the scenario name in the published v1

        sc = self.sc_model.objects.get_versionedobject(v_id=self.slscenario.v_id, version=v1)
        self.assertEquals(sc.name, u'sl scenario')

    def test_suggestionlist_views(self):

        base_url = '/analyses/%s/suggestionlists/' % self.namespace
        defaultuser = User.objects.get(pk=1)
        users = (0,1,2,3)

        class suggestionlists(utils_tests.tablelist_TestDefinition):
            description = 'list of suggestionlists'
            url = base_url
            users = [1,2,]
            rowcount = 2
            contains = [{'url': base_url + 'add/', 'users': [1,]},]

        class suggestionlist_suggestionlist(utils_tests.TestDefinition):
            description = 'display suggestionlist 1 tab suggestionlist'
            url = base_url + '%s/suggestionlist/' % self.suggestionlist.v_id
            users = [1,2,]
            contains = [
                {'url': base_url + '%s/edit/' % self.suggestionlist.v_id, 'users': [1,]},
                {'url': base_url + '%s/delete/' % self.suggestionlist.v_id, 'users': []},
            ]

        class suggestionlist_empty_suggestionlist(utils_tests.TestDefinition):
            description = 'display empty suggestionlist tab suggestionlist'
            url = base_url + '%s/suggestionlist/' % self.emptysuggestionlist.v_id
            users = [1,2,]
            contains = [
                {'url': base_url + '%s/edit/' % self.emptysuggestionlist.v_id, 'users': [1,]},
                {'url': base_url + '%s/delete/' % self.emptysuggestionlist.v_id, 'users': [1,]},
            ]

        class suggestionlist_scenarios(utils_tests.TestDefinition):
            description = 'display suggestionlist tab scenarios'
            url = base_url + '%s/scenarios/' % self.suggestionlist.v_id
            users = [1,2,]
            contains = [
                {'url': base_url + '%s/edit/' % self.suggestionlist.v_id, 'users': []},
                {'url': base_url + '%s/delete/' % self.suggestionlist.v_id, 'users': []},
            ]

        class suggestionlist_compare(utils_tests.TestDefinition):
            description = 'compare suggestionlist'
            url = base_url + '%s/compare/0/' % self.suggestionlist.v_id
            users = [1,2,]

        class suggestionlist_report(utils_tests.edit_TestDefinition):
            description = 'pdf report for a suggestionlist'
            url = base_url + '%s/pdf/' % self.suggestionlist.v_id
            users = [1,2,]
            data = {
                "submit_pdf": "Generate pdf",
            }

        class suggestionlist_add(utils_tests.add_TestDefinition):
            description = 'add suggestionlist'
            url = base_url + 'add/'
            users = [1,]
            data = {
                "description": "",
                "management-referer": base_url
            }
            redirect_url = base_url
            def posttest(self, testcase, allowed):
                if allowed:
                    self.test_class.sl_model.objects.order_by('-id')[0].delete(testcase.user, override=True)

        class suggestionlist_edit(utils_tests.edit_TestDefinition):
            description = 'edit suggestionlist'
            url = base_url + '%s/edit/' % self.suggestionlist.v_id
            users = [1,]
            data = {
                "name": "suggestionlist",
                "description": "",
                "management-referer": base_url + '%s/suggestionlist/' % self.suggestionlist.v_id
            }
            redirect_url = base_url + '%s/suggestionlist/' % self.suggestionlist.v_id

        class suggestionlist_delete(utils_tests.delete_TestDefinition):
            description = 'delete suggestionlist'
            users = [1,]
            redirect_url = base_url
            def pretest(self, testcase, allowed):
                # create a new item that we can delete
                newitem = self.test_class.sl_model(name='to_delete')
                newitem.save(defaultuser)
                self.pk = newitem.pk
                self.url = base_url + '%s/delete/' % newitem.v_id
            def posttest(self, testcase, allowed):
                # clean up if the item was not deleted
                if not allowed:
                    self.test_class.sl_model.objects.get(pk=self.pk).delete(defaultuser, override=True)

        class suggestionlists_for_scope_multiple(utils_tests.TestDefinition):
            description = 'display suggestionlists for scope multiple'
            url = '/analyses/%s/scopes/%s/suggestionlists/' % (self.namespace, self.scope.v_id)
            users = [1,]
            rowcount = 2
            def pretest(self, testcase, allowed):
                # create a new item that we can delete
                newitem = self.test_class.sl_model(name='second SL')
                newitem.save(defaultuser)
                self.pk = newitem.pk
            def posttest(self, testcase, allowed):
                # clean up
                self.test_class.sl_model.objects.get(pk=self.pk).delete(defaultuser, override=True)

        class suggestionlists_for_scope_single(utils_tests.TestDefinition):
            description = 'display suggestionlists for scope single'
            url = '/analyses/%s/scopes/%s/suggestionlists/' % (self.namespace, self.scope.v_id)
            users = [1,]
            redirect_url = '/analyses/%s/scopes/%s/suggestionlist/%s/' % (self.namespace, self.scope.v_id, self.suggestionlist.v_id)

        class suggestionlist_for_scope(utils_tests.edit_TestDefinition):
            description = 'display suggestionlist for scope'
            url = '/analyses/%s/scopes/%s/suggestionlist/%s/' % (self.namespace, self.scope.v_id, self.suggestionlist.v_id)
            users = [1,]
            data = {
                "scenarios-TOTAL_FORMS": "1",
                "scenarios-INITIAL_FORMS": "1",
                "scenarios-MAX_NUM_FORMS": "0",
                "scenarios-0-select": True,
                "scenarios-0-id": self.slscenario.id,
                "management-referer": '/installations/scopes/%s/scenarios/' % self.scope.v_id
            }
            redirect_url = '/installations/sections/%s/scenarios/' % self.scope.get_auto_section().v_id
            def pretest(self, testcase, allowed):
                self.sc_c = self.test_class.sc_model.objects.count()
            def extra_tests(self, testcase, response, allowed):
                if allowed:
                    testcase.assertEquals(self.test_class.sc_model.objects.count(), self.sc_c + 1)
                else:
                    testcase.assertEquals(self.test_class.sc_model.objects.count(), self.sc_c)
            def posttest(self, testcase, allowed):
                # clean up
                if allowed:
                    self.test_class.sc_model.objects.order_by('-id')[0].delete(defaultuser, override=True)

        class scenario_add(utils_tests.add_TestDefinition):
            description = 'add scenario to suggestionlist'
            url = base_url + '%s/scenarios/add/' % self.suggestionlist.v_id
            users = [1,]
            data = {
                "name": "bogus",
                "type": "High pressure",
                "description": "",
                "evaluation": "",
                "management-referer": '/whatever/',
                "consequence": "Kaboum",
            }
            def pretest(self, testcase, allowed):
                newvid = self.test_class.sc_model.objects.next_v_id()
                self.redirect_url = '/analyses/%s/scenarios/%s/causetree/' % (self.test_class.namespace, newvid)
            def posttest(self, testcase, allowed):
                if allowed:
                    self.test_class.sc_model.objects.order_by('-id')[0].delete(testcase.user, override=True)

        class suggestionlist_publish(utils_tests.add_TestDefinition):
            description = 'publish suggestion list'
            url = base_url + '%s/publish/' % self.suggestionlist.v_id
            users = [1,]
            data = {
                "name": "public version suggestionlist",
                "management-referer": base_url
            }
            redirect_url = base_url

        class allsuggestionlists_report(utils_tests.edit_TestDefinition):
            description = 'pdf report for all suggestionlists'
            url = base_url = '/analyses/%s/allsuggestionlists/pdf/' % self.namespace
            users = [1,2,]
            data = {
                "submit_pdf": "Generate pdf",
            }

        class main_allsuggestionlists_report(utils_tests.edit_TestDefinition):
            description = 'pdf report for all suggestionlists'
            url = base_url = '/analyses/allsuggestionlists/pdf/'
            users = [1,2,3]
            data = {
                "submit_pdf": "Generate pdf",
            }

        tests = [
            suggestionlists(self),
            suggestionlist_suggestionlist(self),
            suggestionlist_empty_suggestionlist(self),
            suggestionlist_scenarios(self),
            suggestionlist_compare(self),
            suggestionlist_report(self),
            suggestionlist_add(self),
            suggestionlist_edit(self),
            suggestionlist_delete(self),
            suggestionlists_for_scope_multiple(self),
            suggestionlists_for_scope_single(self),
            suggestionlist_for_scope(self),
            scenario_add(self),
            suggestionlist_publish(self),
            allsuggestionlists_report(self),
            main_allsuggestionlists_report(self),
        ]

        utils_tests.run_tests(self, users, tests)

