"""

Test for versioning of objects

Meant to be subclassed

"""
from django.contrib.auth.models import User
from django.test import TestCase

from installations import models as installations_models

from utils import tests as utils_tests
from utils.versioning.models import WorkVersion, Version

class VersioningTest(TestCase):
    fixtures = ['dummyusers', 'installations']
##    sc_model = Scenario
##    pl_model = ProtectionLayer
##    me_model = Measure
##    ap_model = AttentionPoint
##    sl_model = Suggestionlist
##    namespace = "degradations"

    def setUp(self):
        self.setPermissions()
        defaultuser = User.objects.get(pk=1)
        self.scope = installations_models.Scope.objects.filter(version__isnull=True)[0]
        self.scenario = self.sc_model(name='scenario', scope=self.scope)
        self.scenario.save(defaultuser)
        self.event = self.ev_model(name='C1')
        el = self.event.save(defaultuser, parent=None, treeelementkwargs={'scenario': self.scenario})
        self.measure = self.me_model(name='PL', type='PL')
        self.measure.save(defaultuser)
        self.protectionlayer = self.pl_model(name='PL')
        self.protectionlayer.save(defaultuser, parent=el, treeelementkwargs={'scenario': self.scenario})
        self.plme = self.plme_model(protectionlayer=self.protectionlayer, measure = self.measure)
        self.plme.save(defaultuser)
        self.attentionpoint = self.ap_model(name='Capacity', type='Specification', measure=self.measure)
        self.attentionpoint.save(defaultuser)

        self.suggestionlist = self.sl_model(name='SL')
        self.suggestionlist.save(defaultuser)

    def test_versioning(self):
        defaultuser = User.objects.get(pk=1)

        self.assertEquals(self.sl_model.objects.count(), 1)
        self.assertEquals(self.sc_model.objects.count(), 1)
        self.assertEquals(self.tree_model.objects.count(), 2)
        self.assertEquals(self.ev_model.objects.count(), 1)
        self.assertEquals(self.pl_model.objects.count(), 1)
        self.assertEquals(self.me_model.objects.count(), 1)
        self.assertEquals(self.plme_model.objects.count(), 1)
        self.assertEquals(self.ap_model.objects.count(), 1)

        v1 = Version(name='v1', published=True)
        v1.save(defaultuser)

        self.assertEquals(self.sl_model.objects.count(), 2)
        self.assertEquals(self.sc_model.objects.count(), 2)
        self.assertEquals(self.tree_model.objects.count(), 4)
        self.assertEquals(self.ev_model.objects.count(), 2)
        self.assertEquals(self.pl_model.objects.count(), 2)
        self.assertEquals(self.me_model.objects.count(), 2)
        self.assertEquals(self.plme_model.objects.count(), 2)
        self.assertEquals(self.ap_model.objects.count(), 2)

        a1 = self.sc_model.objects.get_versionedobject(v_id = self.scenario.v_id, version=WorkVersion)

        # Change something in scenario a1
        pl1 = self.pl_model.objects.get_versionedobject(v_id = self.protectionlayer.v_id, version=WorkVersion)
        pl1.name = 'PL1a'
        pl1.save(defaultuser)

        # and something in a suggestionlist
        a2 = self.sl_model.objects.get_versionedobject(v_id = self.suggestionlist.v_id, version=WorkVersion)
        a2.description = 'A2a'
        a2.save(defaultuser)

        self.assertEquals(self.sl_model.objects.dirty().count(), 1)
        self.assertEquals(self.sl_model.objects.filter(published_dirty=True, version__isnull=True).count(), 1)

        self.assertEquals(self.pl_model.objects.dirty().count(), 1)
        self.assertEquals(self.pl_model.objects.filter(published_dirty=True, version__isnull=True).count(), 1)

        self.assertEquals(a1.may_publish(defaultuser), True)

        v2 = Version(name='v2', published=True)
        v2.save(defaultuser, propagate=False)
        a2.publish(defaultuser, v2)

        self.assertEquals(self.sl_model.objects.dirty().count(), 0)
        self.assertEquals(self.sl_model.objects.filter(published_dirty=True, version__isnull=True).count(), 0)

        self.assertEquals(self.pl_model.objects.dirty().count(), 1)
        self.assertEquals(self.pl_model.objects.filter(published_dirty=True, version__isnull=True).count(), 1)

        # let's check the PL in the published v2
        pl1 = self.pl_model.objects.get_versionedobject(v_id=1, version=v2)
        self.assertEquals(pl1.name, 'PL')


        # Check restoring

        pl1 = self.pl_model.objects.get_versionedobject(v_id=1, version=WorkVersion)
        self.assertEquals(pl1.name, 'PL1a')
        a2 = self.sl_model.objects.get_versionedobject(v_id = self.suggestionlist.v_id, version=WorkVersion)
        self.assertEquals(a2.description, 'A2a')

        v1.restore(defaultuser)

        pl1 = self.pl_model.objects.get_versionedobject(v_id=1, version=WorkVersion)
        self.assertEquals(pl1.name, 'PL')
        a2 = self.sl_model.objects.get_versionedobject(v_id = self.suggestionlist.v_id, version=WorkVersion)
        self.assertEquals(a2.description, None)


# TODO: complete this

