from django import forms
from django.contrib.auth.models import User, Group, Permission
from django.contrib.contenttypes.models import ContentType
from django.db.models import Q
from django.utils.translation import ugettext_lazy as _

from comments.models import Comment
from utils import fields as utils_fields
from utils import lists as utils_lists
from utils.versioning import versionedforms

class CommentForm(versionedforms.VersionedForm):
    class Meta:
        model = Comment
        fields = ('comment',)
        attrs = {
            'comment': {'rows': 5, 'cols': 60}
        }

class CommentFullForm(versionedforms.VersionedForm):
    class Meta:
        model = Comment
        fields = ('comment', 'response',)
        attrs = {
            'comment': {'rows': 5, 'cols': 60},
            'response': {'rows': 5, 'cols': 60}
        }

class CommentSheet(versionedforms.VersionedSheet):
    content_object = versionedforms.VersionedForeignKeyField(
            data = 'content_object',
            name = 'content_object',
            label = _('Related object'),
            add_link = True
        )
    class Meta:
        model = Comment
        fields = ('content_object', 'comment', 'user_created', 'date_created', 'response')

    def __init__(self, instance, *args, **kwargs):
        super(CommentSheet, self).__init__(instance, *args, **kwargs)
        if instance:
            self.fields['content_object'].label = instance.content_object.__class__._meta.verbose_name

class CommentFilterForm(utils_lists.FilterForm):
    """Custom filterform for the comments list"""
    comment = forms.CharField(max_length=50, required=False, label=_('Comment'))
    user = utils_fields.UserModelChoiceField(queryset = User.objects.none(), required=False,
        label=_('Comment by'))
    response = forms.ChoiceField(choices=utils_lists.get_filter_choices([(1, _('Without response')), (2, _('With response'))]), required=False, label = _('Response'))

    class Meta:
        attrs = {
            'comment': {'size': 40},
            }

    def __init__(self, *args, **kwargs):
        super(CommentFilterForm, self).__init__(*args, **kwargs)
        _commentpermission = Permission.objects.get_by_natural_key('write_comments', 'comments', 'comment')
        self.fields['user'].queryset = User.objects.filter(
            Q(user_permissions=_commentpermission)|Q(groups__permissions=_commentpermission)
            ).distinct().order_by('last_name', 'first_name')

