from django.test import TestCase
from django.contrib.auth.models import User, AnonymousUser
from utils import tests as utils_tests
from hazards import models as app_models
from utils.versioning.models import Version, WorkVersion
import time

class HazardsTestCase(TestCase):
    fixtures = ['dummyusers','hazards']

    def setUp(self):
        from django.contrib.auth.models import User, Permission
        from django.contrib.contenttypes.models import ContentType
        # give the users permissions

        # authorisations for users:
        # 1 = authorised
        # 2 = unauthorised to edit
        # 3 = unauthorised to even display

        u = User.objects.get(pk=1)

        permissions = [
            ('display_versions','versioning','version'),
            ('edit_versions','versioning','version'),
            ('display_hazards','hazards','substance'),
            ('edit_hazards','hazards','substance'),
        ]
        for item in permissions:
            u.user_permissions.add(Permission.objects.get_by_natural_key(*item))

##        for item in ('substance', 'reaction',):
##            u.user_permissions.add(Permission.objects.get_by_natural_key('change_' + item, 'hazards', item))
##            u.user_permissions.add(Permission.objects.get_by_natural_key('add_' + item, 'hazards', item))
##            u.user_permissions.add(Permission.objects.get_by_natural_key('delete_' + item, 'hazards', item))

        u = User.objects.get(pk=2)
        permissions = [
            ('display_versions','versioning','version'),
            ('display_hazards','hazards','substance'),
        ]
        for item in permissions:
            u.user_permissions.add(Permission.objects.get_by_natural_key(*item))

class HazardsSubstanceTestCase(HazardsTestCase):
    def test_substance_views(self):

        base_url = '/hazards/substances/'
        defaultuser = User.objects.get(pk=1)
        users = (0,1,2,3)

        class hazards(utils_tests.TestDefinition):
            description = 'main hazards view'
            url = '/hazards/'
            users = [1,2,]

        class hazards_publish(utils_tests.add_TestDefinition):
            description = 'publish hazards information'
            url = '/hazards/publish/'
            users = [1,]
            data = {
                "name": "public version",
                "management-referer": base_url
            }
            redirect_url = base_url

        class substances(utils_tests.tablelist_TestDefinition):
            description = 'list of substances'
            url = base_url
            users = [1,2,]
            rowcount = 4
            contains = [{'url': base_url + 'add/', 'users': [1,]},
                {'url': '/hazards/matrix', 'users': [1,2,]},
                {'url': '/hazards/publish/', 'users': [1,]}]

        class substances_filtered(utils_tests.tablelist_filtered_TestDefinition):
            description = 'filtered list of substances'
            url = base_url
            users = [1,2,]
            filterdata = {"filter-name": "A",}
            rowcount = 2

        class substances_filtered_unique(utils_tests.tablelist_filtered_TestDefinition):
            description = 'filtered list of substances with redirect_on_unique'
            url = base_url
            users = [1,2,]
            filterdata = {"filter-name": "FA",}
            redirect_url = base_url + '4/substance/'

        class substances_report_select(utils_tests.TestDefinition):
            description = 'report of list of substances: select options'
            url = base_url + '?submit_pdf=submit_pdf&filter-name=FA'
            users = [1,2,]

        class substances_report(utils_tests.postonly_TestDefinition):
            description = 'pdf of list of substances'
            url = base_url
            users = [1,2,]
            data = {
                "filter": "submit_pdf=submit_pdf&filter-name=FA",
                "submit_pdf": "Generate pdf",
            }

        class substances_csv(utils_tests.postonly_TestDefinition):
            description = 'csv of list of substances'
            url = base_url
            users = [1,2,]
            data = {
                "filter": "filter-name=FA",
                "submit_csv": "Generate csv",
            }

        class substance1_substance(utils_tests.TestDefinition):
            description = 'display substance 1 tab substance'
            # subst 1 is in an interaction: no delete
            url = base_url + '1/substance/'
            users = [1,2,]
            contains = [
                {'url': base_url + '1/edit/', 'users': [1,]},
                {'url': base_url + '1/delete/', 'users': []},
            ]

        class substance4_substance(utils_tests.TestDefinition):
            description = 'display substance 4 tab substance'
            # subst 4 is not in an interaction: delete allowed
            url = base_url + '4/substance/'
            users = [1,2,]
            contains = [
                {'url': base_url + '4/edit/', 'users': [1,]},
                {'url': base_url + '4/delete/', 'users': [1,]},
            ]

        class substance_reactions(utils_tests.TestDefinition):
            description = 'display substance tab reactions'
            url = base_url + '1/reactions/'
            users = [1,2,]
            contains = [
                {'url': base_url + '1/edit/', 'users': []},
                {'url': base_url + '1/delete/', 'users': []},
            ]

        class substance_sections(utils_tests.TestDefinition):
            description = 'display substance tab sections'
            url = base_url + '1/sections/'
            users = [1,2,]
            contains = [
                {'url': base_url + '1/edit/', 'users': []},
                {'url': base_url + '1/delete/', 'users': []},
            ]

        class substance_report(utils_tests.TestDefinition):
            description = 'pdf report for substance'
            url = base_url + '1/pdf/'
            users = [1,2,]

        class substance_compare(utils_tests.TestDefinition):
            description = 'compare substance'
            url = base_url + '1/compare/0/'
            users = [1,2,]

        class substance_add(utils_tests.add_TestDefinition):
            description = 'add a substance'
            url = base_url + 'add/'
            users = [1,]
            data = {
                "name": "E",
                "tag": "",
                "description": "",
                "hazards": "",
                "universal": False,
                "management-referer": base_url
            }
            redirect_url = base_url
            def posttest(self, testcase, allowed):
                if allowed:
                    app_models.Substance.objects.order_by('-id')[0].delete(testcase.user, override=True)

        class substance_edit(utils_tests.edit_TestDefinition):
            description = 'edit a substance'
            url = base_url + '1/edit/'
            users = [1,]
            data = {
                "name": "A",
                "tag": "",
                "description": "",
                "hazards": "",
                "universal": False,
                "management-referer": base_url + '1/substance/'
            }
            redirect_url = base_url + '1/substance/'

        class substance_delete(utils_tests.delete_TestDefinition):
            description = 'delete a substance'
            users = [1,]
            redirect_url = base_url
            def pretest(self, testcase, allowed):
                # create a new item that we can delete
                newitem = app_models.Substance(name='to_delete')
                newitem.save(defaultuser)
                self.pk = newitem.id
                self.url = base_url + '%s/delete/' %newitem.v_id
            def posttest(self, testcase, allowed):
                # clean up if the item was not deleted
                if not allowed:
                    app_models.Substance.objects.get(id=self.pk).delete(defaultuser, override=True)

        class substance_namelookup(utils_tests.TestDefinition):
            description = 'substance name lookup'
            url = base_url + 'name_lookup/'
            users = [1,2,]
            data = {
                "q": "A",
            }
            contains = [
                {'url': 'FA', 'users': [1,2,]},
            ]

        tests = [
            hazards(),
            hazards_publish(),
            substances(),
            substances_filtered(),
            substances_filtered_unique(),
            substances_report_select(),
            substances_report(),
            substances_csv(),
            substance1_substance(),
            substance4_substance(),
            substance_reactions(),
            substance_sections(),
            substance_report(),
            substance_compare(),
            substance_add(),
            substance_edit(),
            substance_delete(),
            substance_namelookup(),
        ]

        utils_tests.run_tests(self, users, tests)

    def test_model_report(self):
        user = User.objects.get(pk=1)
        substance = app_models.Substance.objects.get(pk=1)
        self.assertTrue('\n'.join(substance.get_report_body(user, 'en')).find('Substance'))
        self.assertTrue('\n'.join(substance.get_report_body(user, 'nl')).find('Stof'))

class HazardsReactionTestCase(HazardsTestCase):
    def test_reaction_views(self):

        base_url = '/hazards/reactions/'
        defaultuser = User.objects.get(pk=1)
        users = (0,1,2,3)

        class reactions(utils_tests.tablelist_TestDefinition):
            description = 'list of reactions'
            url = base_url
            users = [1,2,]
            rowcount = 3
            contains = [{'url': base_url + 'add/', 'users': [1,]},
                {'url': '/hazards/matrix', 'users': [1,2,]},
                {'url': '/hazards/publish/', 'users': [1,]}]

        class reactions_filtered(utils_tests.tablelist_filtered_TestDefinition):
            description = 'filtered list of reactions'
            url = base_url
            users = [1,2,]
            filterdata = {"filter-name": "R",}
            rowcount = 3

        class reactions_filtered_unique(utils_tests.tablelist_filtered_TestDefinition):
            description = 'filtered list of reactions with redirect_on_unique'
            url = base_url
            users = [1,2,]
            filterdata = {"filter-name": "RA",}
            redirect_url = base_url + '1/reaction/'

        class reactions_report_select(utils_tests.TestDefinition):
            description = 'report of list of reactions: select options'
            url = base_url + '?submit_pdf=submit_pdf&filter-name=RA'
            users = [1,2,]

        class reactions_report(utils_tests.postonly_TestDefinition):
            description = 'pdf of list of reactions'
            url = base_url
            users = [1,2,]
            data = {
                "filter": "submit_pdf=submit_pdf&filter-name=RA",
                "submit_pdf": "Generate pdf",
            }

        class reactions_csv(utils_tests.postonly_TestDefinition):
            description = 'csv of list of reactions'
            url = base_url
            users = [1,2,]
            data = {
                "filter": "filter-name=RA",
                "submit_csv": "Generate csv",
            }

        class reaction_reaction(utils_tests.TestDefinition):
            description = 'display reaction tab reaction'
            url = base_url + '1/reaction/'
            users = [1,2,]
            contains = [
                {'url': base_url + '1/edit/', 'users': [1,]},
                {'url': base_url + '1/delete/', 'users': [1,]},
            ]

        class reaction_sections(utils_tests.TestDefinition):
            description = 'display reaction tab sections'
            url = base_url + '1/sections/'
            users = [1,2,]
            contains = [
                {'url': base_url + '1/edit/', 'users': []},
                {'url': base_url + '1/delete/', 'users': []},
            ]

        class reaction_compare(utils_tests.TestDefinition):
            description = 'compare a reaction'
            url = base_url + '1/compare/0/'
            users = [1,2,]

        class reaction_report(utils_tests.TestDefinition):
            description = 'pdf report for a reaction'
            url = base_url + '1/pdf/'
            users = [1,2,]

        class reaction_add(utils_tests.add_TestDefinition):
            description = 'add a reaction'
            url = base_url + 'add/'
            users = [1,]
            data = {
                "name": "RD",
                "substance1": 1,
                "substance2": 1,
                "management-referer": base_url
            }
            redirect_url = base_url
            def posttest(self, testcase, allowed):
                if allowed:
                    app_models.Reaction.objects.order_by('-id')[0].delete(testcase.user, override=True)

##            def extra_tests(self, testcase, response, allowed):
##                print response

        class reaction_edit(utils_tests.edit_TestDefinition):
            description = 'edit a reaction'
            url = base_url + '1/edit/'
            users = [1,]
            data = {
                "name": "RA",
                "substance1": 1,
                "substance2": 1,
                "management-referer": base_url + '1/reaction/'
            }
            redirect_url = base_url + '1/reaction/'

        class reaction_delete(utils_tests.delete_TestDefinition):
            description = 'delete a reaction'
            users = [1,]
            redirect_url = base_url
            def pretest(self, testcase, allowed):
                # create a new item that we can delete
                newitem = app_models.Reaction(name='to_delete', substance1 = 1, substance2 = 2)
                newitem.save(defaultuser)
                self.pk = newitem.id
                self.url = base_url + '%s/delete/' % newitem.v_id
            def posttest(self, testcase, allowed):
                # clean up if the item was not deleted
                if not allowed:
                    app_models.Reaction.objects.get(id=self.pk).delete(defaultuser, override=True)

        class reaction_namelookup(utils_tests.TestDefinition):
            description = 'reaction name lookup'
            url = base_url + 'name_lookup/'
            users = [1,2,]
            data = {
                "q": "R",
            }
            contains = [
                {'url': 'RA', 'users': [1,2,]},
            ]

        class reaction_matrix(utils_tests.TestDefinition):
            description = 'display the raction matrix'
            url = '/hazards/matrix/'
            users = [1,2,]

        class reaction_matrix_report(utils_tests.TestDefinition):
            description = 'print the raction matrix'
            url = '/hazards/matrix/pdf/'
            users = [1,2,]

        tests = [
            reactions(),
            reactions_filtered(),
            reactions_filtered_unique(),
            reactions_report_select(),
            reactions_report(),
            reactions_csv(),
            reaction_reaction(),
            reaction_sections(),
            reaction_compare(),
            reaction_report(),
            reaction_add(),
            reaction_edit(),
            reaction_delete(),
            reaction_namelookup(),
            reaction_matrix(),
            reaction_matrix_report(),
        ]

        utils_tests.run_tests(self, users, tests)

    def test_model_report(self):
        user = User.objects.get(pk=1)
        reaction = app_models.Reaction.objects.get(pk=1)
        self.assertTrue('\n'.join(reaction.get_report_body(user, 'en')).find('Reaction'))
        self.assertTrue('\n'.join(reaction.get_report_body(user, 'nl')).find('Reactie'))

class HazardsVersioningTestCase(TestCase):
    fixtures = ['dummyusers','hazards_versioning']

    def setUp(self):
        from django.contrib.auth.models import User, Permission
        from django.contrib.contenttypes.models import ContentType
        # give the users permissions

        # authorisations for users:
        # 1 = authorised
        # 2 = authorised to view all versions
        # 3 = authorised to view public versions
        # 4 = unauthorised

        u = User.objects.get(pk=1)
        permissions = [
            ('edit_versions','versioning','version'),
            ('display_versions','versioning','version'),
            ('display_hazards','hazards','substance'),
        ]
        for item in permissions:
            u.user_permissions.add(Permission.objects.get_by_natural_key(*item))

        u = User.objects.get(pk=2)
        permissions = [
            ('display_versions','versioning','version'),
            ('display_hazards','hazards','substance'),
        ]
        for item in permissions:
            u.user_permissions.add(Permission.objects.get_by_natural_key(*item))

        u = User.objects.get(pk=3)
        permissions = [
            ('display_published_versions','versioning','version'),
            ('display_hazards','hazards','substance'),
        ]
        for item in permissions:
            u.user_permissions.add(Permission.objects.get_by_natural_key(*item))

class SubstanceVersioningTestCase(HazardsVersioningTestCase):
    def test_substance_versioning(self):
        user = User.objects.get(pk=1)

        # current state: two substances unversioned and dirty
        self.assertEquals(app_models.Substance.objects.count(), 2)
        self.assertEquals(app_models.Substance.objects.dirty().count(), 2)

        v1 = Version(name='v1')
        v1.save(user)

        # current state: two substances unversioned, two versioned
        self.assertEquals(app_models.Substance.objects.count(), 4)
        self.assertEquals(app_models.Substance.objects.dirty().count(), 0)

        self.assertEquals(app_models.Substance.objects.next_v_id(), 3)

        # let's add a substance
        newsubst = app_models.Substance(name='C')
        newsubst.save(user)
        self.assertEquals(newsubst.v_id, 3)

        self.assertEquals(app_models.Substance.objects.count(), 5)
        self.assertEquals(app_models.Substance.objects.dirty().count(), 1)

        # wait a bit to distinguish the datetimes on the versions
        time.sleep(0.01)
        v2 = Version(name='v2')
        v2.save(user)

        self.assertEquals(app_models.Substance.objects.count(), 6)
        self.assertEquals(app_models.Substance.objects.dirty().count(), 0)

        # let's change B
        s = app_models.Substance.objects.get_versionedobject(name='B', version=WorkVersion)
        s.tag = 'newCAS'
        s.save(user)

        self.assertEquals(app_models.Substance.objects.count(), 6)
        self.assertEquals(app_models.Substance.objects.dirty().count(), 1)

        # wait a bit to distinguish the datetimes on the versions
        time.sleep(0.01)
        # v3 is a public version
        v3 = Version(name='v3', published=True)
        v3.save(user, propagate=False)
        app_models.publish_hazards(user, v3)

        self.assertEquals(app_models.Substance.objects.count(), 9)
        self.assertEquals(app_models.Substance.objects.dirty().count(), 0)

        # let's delete A
        s = app_models.Substance.objects.get_versionedobject(name='A', version=WorkVersion)
        s.delete(user)

        self.assertEquals(app_models.Substance.objects.count(), 9)
        self.assertEquals(app_models.Substance.objects.dirty().count(), 1)
        self.assertEquals(app_models.Substance.objects.filter(deleted=True).count(), 1)

        # wait a bit to distinguish the datetimes on the versions
        time.sleep(0.01)
        v4 = Version(name='v4')
        v4.save(user)

        self.assertEquals(app_models.Substance.objects.count(), 9)
        self.assertEquals(app_models.Substance.objects.dirty().count(), 0)
        self.assertEquals(app_models.Substance.objects.filter(deleted=True).count(), 1)

        # let's add a substance and delete it again
        newsubst = app_models.Substance(name='D')
        newsubst.save(user)
        self.assertEquals(newsubst.v_id, 4)
        self.assertEquals(app_models.Substance.objects.count(), 10)
        self.assertEquals(app_models.Substance.objects.dirty().count(), 1)
        s = app_models.Substance.objects.get_versionedobject(name='D', version=WorkVersion)
        s.delete(user)
        # D should dissappear entirely
        self.assertEquals(app_models.Substance.objects.count(), 9)
        self.assertEquals(app_models.Substance.objects.dirty().count(), 0)


        # now let's check the manager

        qs = app_models.Substance.objects.all().get_versionedobjects(WorkVersion).order_by('name')
        # B and C
        self.assertEquals(qs.count(), 2)
        self.assertEquals(qs[0].name, 'B')
        self.assertEquals(qs[0].tag, 'newCAS')
        self.assertEquals(qs[1].name, 'C')

        qs = app_models.Substance.objects.all().get_versionedobjects(v1).order_by('name')
        # original A and B
        self.assertEquals(qs.count(), 2)
        self.assertEquals(qs[0].name, 'A')
        self.assertEquals(qs[1].name, 'B')
        self.assertEquals(qs[1].tag, 'oldCAS')

        qs = app_models.Substance.objects.all().get_versionedobjects(v2).order_by('name')
        # A,B and C
        self.assertEquals(qs.count(), 3)
        self.assertEquals(qs[0].name, 'A')
        self.assertEquals(qs[1].name, 'B')
        self.assertEquals(qs[1].tag, 'oldCAS')
        self.assertEquals(qs[2].name, 'C')

        qs = app_models.Substance.objects.all().get_versionedobjects(v3).order_by('name')
        # A,B and C
        self.assertEquals(qs.count(), 3)
        self.assertEquals(qs[0].name, 'A')
        self.assertEquals(qs[1].name, 'B')
        self.assertEquals(qs[1].tag, 'newCAS')
        self.assertEquals(qs[2].name, 'C')

        qs = app_models.Substance.objects.all().get_versionedobjects(v4).order_by('name')
        # B and C
        self.assertEquals(qs.count(), 2)
        self.assertEquals(qs[0].name, 'B')
        self.assertEquals(qs[0].tag, 'newCAS')
        self.assertEquals(qs[1].name, 'C')

        # check collect_items
        self.assertEquals(len(v1.collect_items(user)), 2) # 2 substances
        self.assertEquals(len(v2.collect_items(user)), 1)
        self.assertEquals(len(v3.collect_items(user)), 3)
        self.assertEquals(len(v4.collect_items(user)), 1)

        # now lets test the views on these
        base_url = '/hazards/substances/'
        defaultuser = User.objects.get(pk=1)
        users = (0,1,2,3,4)

        class substances(utils_tests.tablelist_TestDefinition):
            description = 'list of substances'
            url = base_url
            users = [1,2,3]
            def extra_tests(self, testcase, response, allowed):
                # test rowcount here, should be different for user 3
                if testcase.user:
                    if testcase.user.id in [1,2]:
                        testcase.assertEquals(len(response.context['table'].rows), 2)
                    elif testcase.user.id in [3,]:
                        testcase.assertEquals(len(response.context['table'].rows), 3)

        tests = [
            substances(),
        ]

        utils_tests.run_tests(self, users, tests)

        # test restoring a version:

        v2.restore(user)

        qs = app_models.Substance.objects.all().get_versionedobjects(WorkVersion).order_by('name')
        # A,B and C
        self.assertEquals(qs.count(), 3)
        self.assertEquals(qs[0].name, 'A')
        self.assertEquals(qs[1].name, 'B')
        self.assertEquals(qs[1].tag, 'oldCAS')
        self.assertEquals(qs[2].name, 'C')


class HazardsSearchTestCase(HazardsTestCase):
    def test_searches(self):
        users = (0,1,2,3,4)

        class base_search(utils_tests.TestDefinition):
            description = ''
            searchclass = None
            searchitems = []
            users = [1,2,]
            def runtests(self, testcase, allowed, contains, notcontains):
                user = testcase.user or AnonymousUser()
                for item in self.searchitems:
                    res = allowed and item['result'] or 0
                    testcase.assertEquals(len(self.searchclass().search(item['query'], user, WorkVersion, None)), res)

        class search_substance(base_search):
            description = 'search substance'
            searchclass = app_models.Hazards_SubstanceSearch
            searchitems = [
                { "query": 'B', "result": 1},
                { "query": 'D', "result": 0},
            ]

        class search_reaction(base_search):
            description = 'search reaction'
            searchclass = app_models.Hazards_ReactionSearch
            searchitems = [
                { "query": 'B', "result": 1},
                { "query": 'D', "result": 0},
            ]

        tests = [
            search_substance(self),
            search_reaction(self),
        ]

        utils_tests.run_tests(self, users, tests)
