"""
insert doctext here

"""

from copy import deepcopy
from django.utils.datastructures import SortedDict
from django.utils.safestring import mark_safe
from django.forms.widgets import Media

from fields import formfield_callback as defaultformfield_callback
from fields import SheetField

# Sheet

class BaseSheet(object):
    def __init__(self, instance = None, omit_empty=False, report=False, *args, **kwargs):
        self.instance = instance
        self.omit_empty = omit_empty
        self.report= report
        self.fields = deepcopy(self.base_fields)
        self.media = Media()

    def __unicode__(self):
        return self.as_html()

    def as_html(self):
        output = []
        for fieldset in self.fieldsets:
            output.append(fieldset.render(self, self.instance))
        for name, field in self.fields.items():
            output.append(field.render(self, self.instance))
        return mark_safe(u'\n'.join(output))

    def fields(self):
        return self.fields

    def __getitem__(self, name):
        field = self.fields.get(name)
        if field:
            return BoundField(field, self, self.instance)
        return super(BaseSheet, self).__getitem__(name)

class SheetOptions(object):
    def __init__(self, options=None):
        self.model = getattr(options, 'model', None)
        self.fields = getattr(options, 'fields', None)
        self.exclude = getattr(options, 'exclude', None)
        self.fieldsets = getattr(options, 'fieldsets', [])
        self.links = getattr(options, 'links', [])

def get_declared_fields(bases, attrs, with_base_fields=True):
    fields = [(field_name, attrs.pop(field_name)) for field_name, obj in attrs.items() if isinstance(obj, SheetField)]
    fields.sort(lambda x, y: cmp(x[1].creation_counter, y[1].creation_counter))

    if with_base_fields:
        for base in bases[::-1]:        # Reorder fields
            if hasattr(base, 'base_fields'):
                fields = base.base_fields.items() + fields
    else:
        for base in bases[::-1]:
            if hasattr(base, 'declared_fields'):
                fields = base.declared_fields.items() + fields
    return SortedDict(fields)

class SheetMetaclass(type):
    def __new__(cls, name, bases, attrs):
        try:
            parents = [b for b in bases if issubclass(b, Sheet)]
        except NameError:
            parents = None

        ff_cb = attrs.pop('formfield_callback', None)
        if not ff_cb and parents:
            for base in parents[::-1]:
                if hasattr(base, 'formfield_callback'):
                    ff_cb = base.__dict__['formfield_callback']
                    break
        if not parents:
            return super(SheetMetaclass, cls).__new__(cls, name, bases,
                    attrs)

        new_class = type.__new__(cls, name, bases, attrs)
        if ff_cb:
            new_class.formfield_callback = ff_cb
        declared_fields = get_declared_fields(bases, attrs, False)
        opts = new_class._meta = SheetOptions(getattr(new_class, 'Meta', None))
        if opts.model:
            # If a model is defined, extract form fields from it.
            fields = fields_for_model(opts.model, fields=opts.fields,
                                      exclude=opts.exclude, links=opts.links, formfield_callback=ff_cb)
            # Override default model fields with any custom declared ones
            # (plus, include all the other declared fields).
            fields.update(declared_fields)
        else:
            fields = declared_fields

        fieldsets = []
        if opts.fieldsets:
            # Fieldsets
            for fieldset in opts.fieldsets:
                fsfields = []
                removed = []
                for fname in fieldset['fields']:
                    f = fields.get(fname)
                    if f:
                        fsfields.append((fname,f))
                fs = Fieldset(label=fieldset['label'], fields=SortedDict(fsfields))
                fieldsets.append(fs)
            # do not mix fields and fieldsets
            fields = SortedDict()
        else:
            # Reorder fields
            if opts.fields:
                flds = []
                for f in opts.fields:
                    if f in fields:
                        flds.append(f)
                for f in fields:
                    if not f in flds:
                        flds.append(f)
                fields.keyOrder = flds
        new_class.fieldsets = fieldsets
        new_class.declared_fields = declared_fields
        new_class.base_fields = fields
        return new_class

def fields_for_model(model, fields=None, exclude=None, links=[], formfield_callback=None):
    formfield_callback = formfield_callback or defaultformfield_callback
    field_list = []
    opts = model._meta
    for f in opts.fields + opts.many_to_many:
        if fields and not f.name in fields:
            continue
        if exclude and f.name in exclude:
            continue
        formfield = formfield_callback(f, add_link=(f.name in links))
        if formfield:
            field_list.append((f.name, formfield))
    return SortedDict(field_list)

class Sheet(BaseSheet):
    __metaclass__ = SheetMetaclass

class Fieldset(object):
    def __init__(self, label = u'', fields=[], *args, **kwargs):
        self.label = label
        self.fields = fields

    def render(self, sheet, instance):
        output = []
        if self.label:
            output.append(u'<tr><th colspan="2" class="fieldset">%s</th></tr>' % self.label)
        for name, field in self.fields.items():
            output.append(field.render(sheet, instance))
        return mark_safe(u'\n'.join(output))

class BoundField(object):
    def __init__(self, field, sheet, instance, *args, **kwargs):
        self.sheetfield = field
        self.sheet = sheet
        self.instance = instance

    def __unicode__(self):
        return mark_safe(self.sheetfield.render(self.sheet, self.instance))

    def __getitem__(self, name):
        if name == 'label':
            return self.render_label()
        if name == 'field':
            return self.render_field()
        return super(BoundField, self).__getitem__(name)

    def render_label(self):
        return mark_safe(self.sheetfield.render_label(self.instance))

    def render_field(self):
        return mark_safe(self.sheetfield.render_field(self.instance))

    def __eq__(self, other):
        if type(other) == BoundField:
            return cmp(self, other) == 0
        else:
            return False

    def __cmp__(self, other):
        if type(other) == BoundField:
            return cmp(self.sheetfield.get_compare_data(self.instance), other.sheetfield.get_compare_data(other.instance))

