"""
Utility functions to create unittests with the test client

Uses class-based test definitions

"""
import sys, inspect
from django.contrib.auth.models import User


class TestDefinition(object):
    """
    Base class for test definitions
    """
    description = ''
    url = ''
    data = {}
    users = []
    template = None
    contains = []
    printresponse=False

    def __init__(self, test_class=None):
        self.test_class=test_class

    def get_data(self):
        return self.data

    def get_redirect_url(self):
        return self.redirect_url

    def run(self, testcase):
        contains = []
        notcontains = []
        for item in self.contains:
            if testcase.user and testcase.user.id in item['users'] or 0 in item['users']:
                contains.append(item['url'])
            else:
                notcontains.append(item['url'])
        if testcase.user and testcase.user.id in self.users or 0 in self.users:
            allowed = True
        else:
            allowed = False

        self.pretest(testcase, allowed)
        self.runtests(testcase, allowed, contains, notcontains)
        self.posttest(testcase, allowed)

    def pretest(self, testcase, allowed):
        pass

    def runtests(self, testcase, allowed, contains, notcontains):
        response = self.run_test_get(testcase, allowed=allowed,
            contains=contains, notcontains=notcontains)
        self.extra_tests(testcase, response, allowed)
        return response

    def extra_tests(self, testcase, response, allowed):
        pass

    def posttest(self, testcase, allowed):
        pass


    def run_test_get(self, testcase, allowed=False, contains=[], notcontains=[]):
        """
        Test a http get of a url
        """
        response = testcase.client.get(self.url, data=self.get_data())
        if self.printresponse:
            print response
        if allowed:
            testcase.assertEqual(response.status_code, 200)
            if self.template:
                testcase.assertTemplateUsed(response, self.template)
            for item in contains:
                testcase.assertContains(response, item)
            for item in notcontains:
                testcase.assertNotContains(response, item)
        else:
            testcase.assertRedirects(response, '/not_allowed/')
        return response

    def run_test_post(self, testcase, allowed=False, contains=[], notcontains=[]):
        """
        Test a http post of a url
        """
        response = testcase.client.post(self.url, self.get_data())
        if self.printresponse:
            print response
        if allowed:
            redirect_url = self.get_redirect_url()
            if redirect_url:
                testcase.assertRedirects(response, redirect_url)
            else:
                testcase.assertEqual(response.status_code, 200)
            for item in contains:
                testcase.assertContains(response, item)
            for item in notcontains:
                testcase.assertNotContains(response, item)
        else:
            testcase.assertRedirects(response, '/not_allowed/')
        return response

# some specific test definition subclasses:

class tablelist_TestDefinition(TestDefinition):
    """
    Test a list view
    """
    rowcount = None
    def extra_tests(self, testcase, response, allowed):
        if allowed and not self.rowcount is None:
            testcase.assertEquals(len(response.context['table'].rows), self.rowcount)

class tablelist_filtered_TestDefinition(TestDefinition):
    """
    Test a list view with a filter using a POST to trigger the filter
    """
    rowcount = None
    filterdata = {}
    redirect_url = ''

    def runtests(self, testcase, allowed, contains, notcontains):
        response = self.run_test_post(testcase, allowed=allowed,
            contains=contains, notcontains=notcontains)
        if allowed and not self.rowcount is None:
            testcase.assertEquals(len(response.context['table'].rows), self.rowcount)
        self.extra_tests(testcase, response, allowed)
        return response

    def get_data(self):
        return self.filterdata

class edit_TestDefinition(TestDefinition):
    """
    Test an edit view: test both GET and POST paths
    """
    data = {}
    redirect_url = ''
    def runtests(self, testcase, allowed, contains, notcontains):
        """
        Watch out, not checking contains
        """
        response = self.run_test_get(testcase, allowed=allowed)
        response = self.run_test_post(testcase, allowed=allowed)
        self.extra_tests(testcase, response, allowed)
        return response

class add_TestDefinition(edit_TestDefinition):
    """
    Test an add view
    """
    pass

class delete_TestDefinition(edit_TestDefinition):
    """
    Test a delete view
    """
    data = {"submit": "submit"}

class postonly_TestDefinition(TestDefinition):
    """
    Test a view that only accepts POST data, but does not respond to GET
    """
    data = {}
    redirect_url = ''
    def runtests(self, testcase, allowed, contains, notcontains):
        response = self.run_test_post(testcase, allowed=allowed,
            contains=contains, notcontains=notcontains)
        self.extra_tests(testcase, response, allowed)
        return response


def run_tests(testcase, users, tests):
    for u in users:
        if u:
            testcase.user = User.objects.get(pk=u)
            res = testcase.client.login(username=testcase.user.username, password='test')
            testcase.assertEqual(res, True)
        else:
            testcase.user = None
            testcase.client.logout()
        for item in tests:
            try:
                item.run(testcase)
            except Exception, exc:
                msg = ' - test "%s" - user %s' % (item.description, testcase.user and testcase.user.id or 'anonymous')
                args = list(exc.args)
                if not args:
                    arg0 = ''
                else:
                    arg0 = args[0]
                arg0 += msg
                exc.args = [arg0] + args[1:]
                raise





# load extra doc tests fo the utils module

from strings import ugettext_tolanguage, add_suffix_to_string, trim_string, count_ident
__test__ = {
    'ugettext_tolanguage': ugettext_tolanguage,
    'add_suffix_to_string': add_suffix_to_string,
    'trim_string': trim_string,
    'count_ident': count_ident,
}